//
//  ViewController.m
//  VideoCameraTest
//
//  Created by Shimoda on 2015/09/13.
//  Copyright © 2015年 Osamu Envision Technology Inc. All rights reserved.
//

#import "ViewController.h"

@interface ViewController ()

@end

@implementation ViewController
@synthesize videoImageView;
@synthesize monoImageView;

- (void)viewDidLoad
{
    [super viewDidLoad];
    // Do any additional setup after loading the view, typically from a nib.
// ビデオキャプチャデバイスの取得
    AVCaptureDevice*    device;
    device = [self frontCamera];
// デバイス入力の取得
    AVCaptureDeviceInput*   deviceInput;
    deviceInput = [AVCaptureDeviceInput deviceInputWithDevice:device error:NULL];
// ビデオデータ出力の作成
    NSMutableDictionary*        settings;
    AVCaptureVideoDataOutput*   dataOutput;
    settings = [NSMutableDictionary dictionary];
    [settings setObject:[NSNumber numberWithInt:kCVPixelFormatType_32BGRA]
                 forKey:(id)kCVPixelBufferPixelFormatTypeKey];
    dataOutput = [[AVCaptureVideoDataOutput alloc] init];
    [dataOutput autorelease];
    dataOutput.videoSettings = settings;
    [dataOutput setSampleBufferDelegate:self queue:dispatch_get_main_queue()];
// Videoキャプチャセッションの作成
    m_AVCapSession = [[AVCaptureSession alloc] init];
    [m_AVCapSession addInput:deviceInput];
    [m_AVCapSession addOutput:dataOutput];
    m_AVCapSession.sessionPreset = AVCaptureSessionPresetMedium;
    
}
- (void)viewDidUnload
{
    [m_AVCapSession release];
}

- (void)didReceiveMemoryWarning
{
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

- (void)dealloc
{
    [videoImageView release];
    [monoImageView release];
    [super dealloc];
}


- (void)captureOutput:(AVCaptureOutput*)captureOutput didOutputSampleBuffer:(CMSampleBufferRef)sampleBuffer fromConnection:(AVCaptureConnection*)connection
{
// イメージバッファの取得
    CVImageBufferRef    buffer;
    buffer = CMSampleBufferGetImageBuffer(sampleBuffer);
    
// イメージバッファのロック
    CVPixelBufferLockBaseAddress(buffer, 0);
    
// イメージバッファ情報の取得
    uint8_t*    base;
    size_t      width, height, bytesPerRow;
    base = CVPixelBufferGetBaseAddress(buffer);
    width = CVPixelBufferGetWidth(buffer);
    height = CVPixelBufferGetHeight(buffer);
    bytesPerRow = CVPixelBufferGetBytesPerRow(buffer);
    
// ビットマップコンテキストの作成
    CGColorSpaceRef colorSpace;
    CGContextRef    cgContext;
    colorSpace = CGColorSpaceCreateDeviceRGB();
    cgContext = CGBitmapContextCreate(
                                      base, width, height, 8, bytesPerRow, colorSpace,
                                      kCGBitmapByteOrder32Little | kCGImageAlphaPremultipliedFirst);
    CGColorSpaceRelease(colorSpace);

    if(m_fMono == TRUE)
    {
        CIImage *ciImage = [CIImage imageWithCVPixelBuffer:buffer options:[NSDictionary dictionaryWithObjectsAndKeys:[NSNull null], kCIImageColorSpace, nil]];
// フィルタを読み込む
        CIFilter *ciFilter = [CIFilter filterWithName:@"CIPhotoEffectMono" keysAndValues:kCIInputImageKey, ciImage, nil];
        
        CGImageRef imageRef = [m_ciContext createCGImage:[ciFilter outputImage] fromRect:[[ciFilter outputImage] extent]];//
        monoImageView.image = [UIImage imageWithCGImage:imageRef scale:1.0f orientation:UIImageOrientationRight];
        CGImageRelease(imageRef);
    }
    
// 画像の作成
    CGImageRef  cgImage;
    UIImage*    image;
    cgImage = CGBitmapContextCreateImage(cgContext);
    image = [UIImage imageWithCGImage:cgImage scale:1.0f
                          orientation:UIImageOrientationRight];
    CGImageRelease(cgImage);
    CGContextRelease(cgContext);
    
// イメージバッファのアンロック
    CVPixelBufferUnlockBaseAddress(buffer, 0);
    
// 画像の表示
    videoImageView.image = image;
}



- (IBAction)StartCapture:(id)sender
{
    m_ciContext = [[CIContext contextWithOptions:nil] retain];
    m_fMono = FALSE;
// Videoキャプチャセッションの開始
    [m_AVCapSession startRunning];
}

- (IBAction)StartCaptureWithCreateCGImage:(id)sender
{
// Memory leak occurs useing createCGImage, CGImageRelease with iOS9
// but not occurs with iOS8
    m_ciContext = [[CIContext contextWithOptions:nil] retain];
    m_fMono = TRUE;
    // Videoキャプチャセッションの開始
    [m_AVCapSession startRunning];
}


- (IBAction)StopCapture:(id)sender
{
// Videoキャプチャセッションの停止
    [m_AVCapSession stopRunning];
    [m_ciContext release];
}

- (AVCaptureDevice *)frontCamera
{
    NSArray *devices = [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo];
    for (AVCaptureDevice *device in devices)
    {
        if ([device position] == AVCaptureDevicePositionFront)
        {
            m_fBackCamera = false;
            return device;
        }
    }
// 正面カメラがない時は、背面カメラを選択
    for (AVCaptureDevice *device2 in devices)
    {
        if ([device2 position] == AVCaptureDevicePositionBack)
        {
            m_fBackCamera = true;
            return device2;
        }
    }
    return nil;
}

- (AVCaptureDevice *)backCamera
{
    NSArray *devices = [AVCaptureDevice devicesWithMediaType:AVMediaTypeVideo];
    
// 正面カメラがない時は、背面カメラを選択
    for (AVCaptureDevice *device2 in devices)
    {
        if ([device2 position] == AVCaptureDevicePositionBack)
        {
            m_fBackCamera = true;
            return device2;
        }
    }
    return nil;
}


@end
